function [h, a, delt_teor]=fir_min_max(M,Omp,Oms,Wp,Ws);
% [h,a,delt_teor]=fir_min_max(M,Omp,Oms,Wp,Ws);
%
% projektowanie optymalnego w sensie minimum bledow maksymalnych 
% dolnoprzepustowego filtra FIR o symetrycznej odpowiedzi 
% impulsowej typu I i dlugosci N=2M+1
% z wazona charakterystyka amplitudowa
% Omp, Oms - pasmo przepustowe i zaporowe w radianach
% Wp, Ws   - wagi w pasmie przepustowym i zaporowym
%
% Przyklad 4.4.
% m-plik (funkcyjny): fir_min_max.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 4. Projektowanie filtrow. 
% Autorzy rozdzialu: Krzysztof Duda, Przemyslaw Korohoda.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2013 r./KD.

N_eks = M+2; %liczba ekstremow
dOm = (Omp(2)+Oms(2)-Oms(1))/N_eks;
Om = 0:dOm:pi;
Np=round(Omp(2)/dOm); %liczba ekstremow w pasmie przepustowym    
Ns=N_eks-Np;          %liczba ekstremow w pasmie zaporowym    
Omk = [Om(1:Np) Om(end-Ns+1:end)]; %czestotliwosci ekstremow
for KK=1:1e2    %petla moze byc przerwana funkcja break w linni 48
    R  = zeros(N_eks,N_eks-1);     
    c  = ones(N_eks,1);
    ind_c= find(Omk>=Oms(1));
    c(ind_c) = 0;    
    znak = (-1).^([1:N_eks]); 
    w = znak/Wp;
    w(ind_c) = znak(ind_c)/Ws;    
    w=w(:);
    for k=0:N_eks-1
        for m=0:M
            R(k+1,m+1)=cos(m*Omk(k+1));
        end
    end    
    a_delt = inv([R w])*c;
%% ch-ka filtra
    Om = 0:pi/1e3:pi;
    Hw = zeros(size(Om));          
    for n=0:M
        Hw = Hw + a_delt(n+1)*cos(n*Om );             
    end
    pochodna = diff(Hw);
    zmiana_znaku = [0 pochodna].*[pochodna 0];
    ind1= find(pochodna==0);
    ind2= find(zmiana_znaku<0);                
    ind = [ind2 ind1]; %indeksy do maksimow   
%% obliczenie bledu
    delt_teor = abs(a_delt(N_eks));
    Om_eks = Om(ind);
    Hw_eks = Hw(ind);
    ind_eks= find(Om_eks>=Oms(1));                
    delt         = Wp*(1-Hw_eks);
    delt(ind_eks)= Ws*(0-Hw_eks(ind_eks)); %pasmo zaporowe        
    delt_max=max(abs(abs(delt)));           
    if abs(delt_max-delt_teor)<1e-9;
        disp(['Wykonano ' num2str(KK) ' iteracji'])
        break   %wyjscie z petli
    end                  
    if Wp>=Ws                        
        Omk = [Om(ind) 0 pi Omp(2) Oms(1)]; %nowe czestotliwosci    
    else
        Omk = [Om(ind) 0 pi Oms(1) Omp(2)]; %nowe czestotliwosci  
    end   
    % Omk = sort( Omk(1:N_eks),'ascend');
    Omk = sort(Omk(1:N_eks));
    Omk=Omk(end:-1:1);
end
for k=1:M
   h(k) = a_delt(M-k+2)/2;  
end
h = [h  a_delt(1) fliplr(h)];

% KONIEC PRZYKLADU 4.4.
