function [h,a]=fir_mse(M,Omc);
% [h,a]=fir_mse(M,Omc);
%
% projektowanie optymalnego w sensie sredniokwadratowym 
% dolnoprzepustowego filtra FIR o symetrycznej odpowiedzi 
% impulsowej typu I i dlugosci N=2M+1
% Omc - krawedz pasma w radianach
%
% Przyklad 4.2.
% m-plik (funkcyjny): fir_mse.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 4. Projektowanie filtrow. 
% Autorzy rozdzialu: Krzysztof Duda, Przemyslaw Korohoda.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2013 r./KD.

dOm= pi/1e3;          %krok calkowania
Om = -pi: dOm : pi-dOm; 
NOm= length(Om);
iR = zeros(M+1,M+1); % macierz iR nie zalezy od wspolczynnikow filtra
iR(1,1)=1;
for k=2:M+1    
    iR(k, k) = 2;
end
OmD = ones(size(Om)); %zadana charakterystyka amplitudowa
ind = find(abs(Om)>Omc);
OmD(ind) = 0;
c  = zeros(M+1,1);
for k=0:M
    c(k+1,1) = sum( OmD.*cos(k*Om) ) / NOm;
end
a = iR*c;
for k=1:M
   h(k) = a(k+1)/2;  
end
h = [fliplr(h) a(1) h];

% KONIEC PRZYKLADU 4.2.